#!/bin/bash


# ibc-backup
#
# Versión: 0.2.2 (11-02-2007)
# Autor: Iñaki Baz Castillo <ibc@aliax.net>
# Licencia: GPL 3




################ Parámetros de configuración ################

# Directorio con los profiles de backup:
dir_profiles="/home/ibc/bin/ibc-backup/profiles"

# Valor por defecto del tiempo de antigüedad máxima para conservar versiones antiguas de ficheros (s, m, h, D, W, M, or Y):
MAX_ANTIGUEDAD="2M"  # 2 meses.

############################################################




# Definimos algunos colores:
ROJO='\e[1;31m'
AZUL='\e[1;34m'
CYAN='\e[1;36m'
VERDE='\e[1;32m'
AMARILLO='\e[1;33m'
NC='\e[0m' # No Color



echo -e "${CYAN}"
echo "******************************"
echo "********* ibc-backup *********"
echo "******************************"


profile="$dir_profiles/$1"  # Fichero con los datos del backup.

echo -e "${ROJO}"  # Para los errores

# Comprobamos si se ha pasado un parámetro:
if [ "$profile" == "$dir_profiles/" ] ; then
	echo -e "\nERROR: Sintaxis de uso:\n\t$0 fichero_profile\n"
	echo -e "${NC}"
	echo -e "\nEstos son los ficheros profile existentes en $dir_profiles:\n"
	echo -e "${AMARILLO}"
	cd $dir_profiles
	for file in * ; do
		echo -e "\t$file"
	done
	echo -e "${NC}"
	exit 1
fi

# Comprobamos si el primer (y en teoría único) parámetro es un archivo válido de backup:
if ! [ -f $profile ] ; then
	echo -e "\nERROR: El fichero $profile no existe.\n"
	echo -e "${NC}"
	echo -e "\nEstos son los ficheros profile existentes en $dir_profiles:\n"
	echo -e "${AMARILLO}"
	cd $dir_profiles
	for file in * ; do
		echo -e "\t$file"
	done
	echo -e "${NC}"
	exit 2
fi

# Leemos las variables:
source $profile

# Comprobamos si las variables están declaradas:
if ! [ "$BASE_ORIGEN" ] || ! [ "$BASE_DESTINO" ] ; then
	echo -e "\nERROR: La variable BASE_ORIGEN y/o BASE_DESTINO no están declaradas.\n"
	echo -e "${NC}"
	exit 3
fi

# Comprobamos si existe y tenemos permisos r y x en BASE_ORIGEN:
## ¡ Lo quito, ya que no sirve si el origen es remoto !
#if ! [ -d $BASE_ORIGEN ] || ! [ -x $BASE_ORIGEN ] || ! [ -r $BASE_ORIGEN ] ; then
#	echo -e "\nERROR: El directorio BASE_ORIGEN '$BASE_ORIGEN' no existe o no tienes permisos de exploración (x) y lectura (r) en él.\n"
#	echo -e "${NC}"
#	exit 4
#fi

# Comprobamos si existe y tenemos permisos r, x y w en BASE_DESTINO:
## ¡ No hace falta que exista gracias a la opción "--create-full-path" (aunque entonces no testeamos si tenemos permisos, ya veremos...).
# if ! [ -d $BASE_DESTINO ] || ! [ -w $BASE_DESTINO ] || ! [ -x $BASE_DESTINO ] || ! [ -r $BASE_DESTINO ] ; then
# 	echo -e "\nERROR: El directorio BASE_DESTINO '$BASE_DESTINO' no existe o no tienes permisos de escritura (w), exploración (x) y lectura (r) en él.\n"
# 	echo -e "${NC}"
# 	exit 5
# fi

# Comprobamos si existe o bien sólo INCLUDE o bien sólo EXCLUDE o ninguna:
if [ "$INCLUDE" ] && [ "$EXCLUDE" ] ; then
	echo -e "\nERROR: Sólo puede estar definida la variable INCLUDE o EXCLUDE o ninguna de ellas, pero no ambas.\nRevise el fichero $profile.\n"
	echo -e "${NC}"
	exit 6
fi


# Si se define INCLUDE entonces linea_exclude lo convertimos en '**'. Excluimos todo menos el INCLUDE:
if [ "$INCLUDE" ] ; then
	linea_exclude="--exclude '**'"
fi

# Si se define EXCLUDE encontes INCLUDE no lo tocamos (no debe ser '**' o da error por ser redundante al copiarse todo por defecto):
# Si no se define ni INCLUDE ni EXCLUDE entonces ocurre lo mismo.


# Visualizamos los datos del backup:
echo -e "${NC}"
echo -e "\nDatos del backup $profile:"
echo -e "${AMARILLO}"
echo -e "\tDESCRIPCION = \t\t$DESCRIPCION"
echo -e "\n\tORIGEN = \t\t$ORIGEN"
echo -e "\n\tBASE_ORIGEN = \t\t$BASE_ORIGEN"
echo -e "\n\tDESTINO = \t\t$DESTINO"
echo -e "\n\tBASE_DESTINO = \t\t$BASE_DESTINO"
echo -e "\n\tINCLUDE" =
for dir in $INCLUDE ; do
	echo -e "\t\t\t\t$BASE_ORIGEN/$dir"
done
echo -e "\n\tEXCLUDE" =
for dir in $EXCLUDE ; do
	echo -e "\t\t\t\t\t$BASE_ORIGEN/$dir"
done
echo -e "\n\tMAX_ANTIGUEDAD = \t$MAX_ANTIGUEDAD"
echo -e "${NC}"
echo -e "\n\n"


# Preparamos el comando y las variables:

comando="rdiff-backup --create-full-path -v 3"

for dir in $INCLUDE ; do
	linea_include="$linea_include --include $BASE_ORIGEN/$dir"
done

for dir in $EXCLUDE ; do
	linea_exclude="$linea_exclude --exclude $BASE_ORIGEN/$dir"
done

if [ "$ORIGEN" ] ; then
	linea_origen="$ORIGEN::$BASE_ORIGEN"
else
	linea_origen="$BASE_ORIGEN"
fi


if [ "$DESTINO" ] ; then
	linea_destino="$DESTINO::$BASE_DESTINO"
else
	linea_destino="$BASE_DESTINO"
fi


# Empezamos el backup:
echo -e "\nRealizando el backup ...\n"
comando_completo="$comando  $linea_include  $linea_exclude  $linea_origen  $linea_destino"
echo -e "${CYAN}"
echo -e "#>  $comando_completo"
echo -e "${NC}"
bash -c "$comando_completo"


if [ $? != 0 ] ; then
	echo -e "${ROJO}"
	echo -e "\nERROR en el backup. Salimos.\n"
	echo -e "${NC}"
	exit 7
fi

echo -e "${VERDE}"
echo -e "\nEXITO en el backup."
echo -e "${NC}"


# Eliminando versiones sin cambios desde hace MAX_ANTIGUEDAD:
echo -e "\nEliminando versiones sin cambios desde hace $MAX_ANTIGUEDAD ...\n"
rdiff-backup --remove-older-than $MAX_ANTIGUEDAD $BASE_DESTINO

if [ $? != 0 ] ; then
	echo -e "${ROJO}"
	echo -e "\nERROR al borrar ficheros antiguos en el backup. Salimos.\n"
	echo -e "${NC}"
	exit 8
fi

echo -e "${VERDE}"
echo -e "\n\nBackup terminado con EXITO.\n"
echo -e "${NC}"

exit 0